@@ -62,6 +62,9 @@
 #define PREVIEW_WIDTH    (PREVIEW_HEIGHT * MAX_ASPECT_RATIO)
 
 
+#define SETTINGS_WINDOW_LAST_WIDTH           "/last/window-width"
+#define SETTINGS_WINDOW_LAST_HEIGHT          "/last/window-height"
+
 #define SHOW_DESKTOP_MENU_PROP               "/desktop-menu/show"
 #define DESKTOP_MENU_SHOW_ICONS_PROP         "/desktop-menu/show-icons"
 
@@ -377,11 +380,11 @@ setup_special_icon_list(GtkBuilder *gxml,
           DESKTOP_ICONS_SHOW_TRASH, TRUE },
         { N_("Removable Devices"), "drive-removable-media", "gnome-dev-removable",
           DESKTOP_ICONS_SHOW_REMOVABLE, TRUE },
-        { N_("Network Shares"), "network_fs", "gnome-dev-network",
+        { N_("Network Shares"), "gtk-network", "gnome-dev-network",
           DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE, TRUE },
         { N_("Disks and Drives"), "drive-harddisk-usb", "gnome-dev-removable-usb",
           DESKTOP_ICONS_SHOW_DEVICE_REMOVABLE, TRUE },
-        { N_("Other Items"), "phone-symbolic", "phone",
+        { N_("Other Devices"), "multimedia-player", "phone",
           DESKTOP_ICONS_SHOW_UNKNWON_REMOVABLE, TRUE },
         { NULL, NULL, NULL, NULL, FALSE },
     };
@@ -557,6 +560,8 @@ cb_destroy_add_dir_enumeration(gpointer user_data)
         g_object_unref(panel->cancel_enumeration);
         panel->cancel_enumeration = NULL;
     }
+
+    panel->add_dir_idle_id = 0;
 }
 
 static gboolean
@@ -687,8 +692,7 @@ xfdesktop_settings_update_iconview_frame_name(AppearancePanel *panel,
             /* This is for the infobar letting the user know how to configure
              * multiple monitor setups */
             gtk_label_set_text(GTK_LABEL(panel->infobar_label),
-                               _("You are using more than one display, "
-                                 "move this dialog to the display you "
+                               _("Move this dialog to the display you "
                                  "want to edit the settings for."));
             gtk_widget_set_visible(panel->infobar, TRUE);
         } else {
@@ -706,8 +710,7 @@ xfdesktop_settings_update_iconview_frame_name(AppearancePanel *panel,
             /* This is for the infobar letting the user know how to configure
              * multiple monitor/workspace setups */
             gtk_label_set_text(GTK_LABEL(panel->infobar_label),
-                               _("You are using more than one display, "
-                                 "move this dialog to the display and "
+                               _("Move this dialog to the display and "
                                  "workspace you want to edit the settings for."));
             gtk_widget_set_visible(panel->infobar, TRUE);
         }
@@ -1077,13 +1080,24 @@ cb_folder_selection_changed(GtkWidget *button,
 
     /* Check to see if the folder actually did change */
     if(g_strcmp0(filename, previous_filename) == 0) {
-        DBG("filename %s, previous_filename %s. Nothing changed",
-            filename == NULL ? "NULL" : filename,
-            previous_filename == NULL ? "NULL" : previous_filename);
+        const gchar *current_folder = xfdesktop_settings_get_backdrop_image(panel);
+        gchar *dirname;
 
-        g_free(filename);
-        g_free(previous_filename);
-        return;
+        dirname = g_path_get_dirname(current_folder);
+
+        /* workaround another gtk bug - if the user sets the file chooser
+         * button to something then it can't be changed with a set folder
+         * call anymore. */
+        if(g_strcmp0(filename, dirname) == 0) {
+            DBG("folder didn't change");
+            g_free(dirname);
+            g_free(filename);
+            g_free(previous_filename);
+            return;
+        } else {
+            g_free(filename);
+            filename = dirname;
+        }
     }
 
     TRACE("folder changed to: %s", filename);
@@ -1188,6 +1202,8 @@ xfdesktop_settings_update_iconview_folder(AppearancePanel *panel)
     current_folder = xfdesktop_settings_get_backdrop_image(panel);
     dirname = g_path_get_dirname(current_folder);
 
+    DBG("current_folder %s, dirname %s", current_folder, dirname);
+
     gtk_file_chooser_set_current_folder((GtkFileChooser*)panel->btn_folder, dirname);
 
     /* Workaround for a bug in GTK */
@@ -1444,11 +1460,6 @@ cb_update_background_tab(WnckWindow *wnck_window,
                 g_free(monitor_name);
                 return;
             }
-        } else {
-            /* Check if we haven't changed monitors (by number) */
-            if(panel->monitor == monitor_num) {
-                return;
-            }
         }
     }
 
@@ -1643,18 +1654,18 @@ xfdesktop_settings_setup_image_iconview(AppearancePanel *panel)
 
 static void
 xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
-                                     XfconfChannel *channel,
-                                     GdkScreen *screen)
+                                     AppearancePanel *panel)
 {
     GtkWidget *appearance_container, *chk_custom_font_size,
               *spin_font_size, *w, *box, *spin_icon_size,
               *chk_show_thumbnails, *chk_single_click, *appearance_settings,
               *chk_show_tooltips, *spin_tooltip_size, *bnt_exit, *content_area;
     GtkBuilder *appearance_gxml;
-    AppearancePanel *panel = g_new0(AppearancePanel, 1);
     GError *error = NULL;
     GtkFileFilter *filter;
+    GdkScreen *screen;
     WnckScreen *wnck_screen;
+    XfconfChannel *channel = panel->channel;
 
     TRACE("entering");
 
@@ -1714,10 +1725,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(chk_show_thumbnails),
                                   TRUE);
 
-    /* Background tab */
-    panel->channel = channel;
-    panel->screen = gdk_screen_get_number(screen);
-
+    screen = gtk_widget_get_screen(appearance_container);
     wnck_screen = wnck_screen_get(panel->screen);
 
     /* watch for workspace changes */
@@ -1931,12 +1939,31 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
 }
 
 static void
-xfdesktop_settings_response(GtkWidget *dialog, gint response_id)
+xfdesktop_settings_response(GtkWidget *dialog, gint response_id, gpointer user_data)
 {
-    if(response_id == GTK_RESPONSE_HELP)
-        xfce_dialog_show_help(GTK_WINDOW(dialog), "xfdesktop", "preferences", NULL);
-    else
+    if(response_id == GTK_RESPONSE_HELP) {
+        xfce_dialog_show_help_with_version(GTK_WINDOW(dialog),
+                                           "xfdesktop",
+                                           "start",
+                                           NULL,
+                                           XFDESKTOP_VERSION_SHORT);
+    } else {
+        XfconfChannel *channel = (XfconfChannel*) user_data;
+        GdkWindowState state;
+        gint width, height;
+
+        /* don't save the state for full-screen windows */
+        state = gdk_window_get_state(GTK_WIDGET(dialog)->window);
+
+        if ((state & (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN)) == 0) {
+            /* save window size */
+            gtk_window_get_size(GTK_WINDOW(dialog), &width, &height);
+            xfconf_channel_set_int(channel, SETTINGS_WINDOW_LAST_WIDTH, width);
+            xfconf_channel_set_int(channel, SETTINGS_WINDOW_LAST_HEIGHT, height);
+        }
+
         gtk_main_quit();
+    }
 }
 
 static GdkNativeWindow opt_socket_id = 0;
@@ -1952,8 +1979,9 @@ main(int argc, char **argv)
 {
     XfconfChannel *channel;
     GtkBuilder *gxml;
-    GdkScreen *screen;
+    gint screen;
     GError *error = NULL;
+    AppearancePanel *panel = g_new0(AppearancePanel, 1);
 
 #ifdef G_ENABLE_DEBUG
     /* do NOT remove this line. If something doesn't work,
@@ -2017,16 +2045,22 @@ main(int argc, char **argv)
         GtkWidget *dialog;
         dialog = GTK_WIDGET(gtk_builder_get_object(gxml, "prefs_dialog"));
         g_signal_connect(dialog, "response",
-                         G_CALLBACK(xfdesktop_settings_response), NULL);
+                         G_CALLBACK(xfdesktop_settings_response),
+                         channel);
+        gtk_window_set_default_size
+            (GTK_WINDOW(dialog),
+             xfconf_channel_get_int(channel, SETTINGS_WINDOW_LAST_WIDTH, -1),
+             xfconf_channel_get_int(channel, SETTINGS_WINDOW_LAST_HEIGHT, -1));
         gtk_window_present(GTK_WINDOW (dialog));
 
-        screen = gtk_widget_get_screen(dialog);
+        screen = gdk_screen_get_number(gtk_widget_get_screen(dialog));
 
         /* To prevent the settings dialog to be saved in the session */
         gdk_x11_set_sm_client_id("FAKE ID");
 
     } else {
         GtkWidget *plug, *plug_child;
+        WnckScreen *wnck_screen;
 
         plug = gtk_plug_new(opt_socket_id);
         gtk_widget_show(plug);
@@ -2039,10 +2073,20 @@ main(int argc, char **argv)
         gtk_widget_reparent(plug_child, plug);
         gtk_widget_show(plug_child);
 
-        screen = gtk_widget_get_screen(plug);
+        screen = gdk_screen_get_number(gtk_widget_get_screen(plug));
+
+        /* In a GtkPlug setting there isn't an easy way to find our window
+         * in cb_window_opened so we'll just force wnck to init and get the
+         * active window */
+        wnck_screen = wnck_screen_get(screen);
+        wnck_screen_force_update(wnck_screen);
+        panel->wnck_window = wnck_screen_get_active_window(wnck_screen);
     }
 
-    xfdesktop_settings_dialog_setup_tabs(gxml, channel, screen);
+    panel->channel = channel;
+    panel->screen = screen;
+
+    xfdesktop_settings_dialog_setup_tabs(gxml, panel);
 
     gtk_main();
 
@@ -2051,5 +2095,7 @@ main(int argc, char **argv)
     g_object_unref(G_OBJECT(channel));
     xfconf_shutdown();
 
+    g_free(panel);
+
     return 0;
 }
