@@ -114,7 +114,6 @@ struct _XfceDesktopPriv
     gboolean icons_font_size_set;
     guint icons_font_size;
     guint icons_size;
-    gint  style_refresh_timer;
     GtkWidget *icon_view;
     gdouble system_font_size;
 #endif
@@ -367,7 +366,10 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     TRACE("entering");
     
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
+
+    if(!XFCE_IS_BACKDROP(backdrop))
+        return;
+
     if(desktop->priv->updates_frozen || !gtk_widget_get_realized(GTK_WIDGET(desktop)))
         return;
 
@@ -604,7 +606,7 @@ workspace_changed_cb(WnckScreen *wnck_screen,
 {
     XfceDesktop *desktop = XFCE_DESKTOP(user_data);
     gint current_workspace, new_workspace, i;
-    XfceBackdrop *new_backdrop;
+    XfceBackdrop *current_backdrop, *new_backdrop;
 
     TRACE("entering");
 
@@ -625,11 +627,16 @@ workspace_changed_cb(WnckScreen *wnck_screen,
         current_workspace, new_workspace);
 
     for(i = 0; i < xfce_desktop_get_n_monitors(desktop); i++) {
-        /* Sanity check */
+        /* We want to compare the current workspace backdrop with the new one
+         * and see if we can avoid changing them if they are the same image/style */
         if(current_workspace < desktop->priv->nworkspaces && current_workspace >= 0) {
-            /* update! */
+            current_backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[current_workspace], i);
             new_backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[new_workspace], i);
-            backdrop_changed_cb(new_backdrop, user_data);
+
+            if(!xfce_backdrop_compare_backdrops(current_backdrop, new_backdrop) || !desktop->priv->bg_pixmap) {
+                /* only update monitors that require it */
+                backdrop_changed_cb(new_backdrop, user_data);
+            }
         } else {
             /* If current_workspace was removed or never existed, get the new
              * backdrop and apply it */
@@ -891,9 +898,6 @@ xfce_desktop_finalize(GObject *object)
     g_object_unref(G_OBJECT(desktop->priv->channel));
     g_free(desktop->priv->property_prefix);
 
-    if(desktop->priv->style_refresh_timer != 0)
-        g_source_remove(desktop->priv->style_refresh_timer);
-
     G_OBJECT_CLASS(xfce_desktop_parent_class)->finalize(object);
 }
 
@@ -1000,14 +1004,11 @@ xfce_desktop_realize(GtkWidget *widget)
     gtk_window_set_screen(GTK_WINDOW(desktop), desktop->priv->gscreen);
     sw = gdk_screen_get_width(desktop->priv->gscreen);
     sh = gdk_screen_get_height(desktop->priv->gscreen);
-    if(gtk_major_version > 2
-       || (gtk_major_version == 2 && gtk_minor_version >= 13))
-    {
-        g_signal_connect(G_OBJECT(desktop->priv->gscreen),
-                         "monitors-changed",
-                         G_CALLBACK(xfce_desktop_monitors_changed),
-                         desktop);
-    }
+
+    g_signal_connect(G_OBJECT(desktop->priv->gscreen),
+                     "monitors-changed",
+                     G_CALLBACK(xfce_desktop_monitors_changed),
+                     desktop);
     
     /* chain up */
     GTK_WIDGET_CLASS(xfce_desktop_parent_class)->realize(widget);
@@ -1214,7 +1215,8 @@ xfce_desktop_popup_menu(GtkWidget *w)
     }
     
     xfce_desktop_popup_root_menu(XFCE_DESKTOP(w), button, etime);
-    
+
+    gdk_event_free((GdkEvent*)evt);
     return TRUE;
 }
 
@@ -1253,26 +1255,45 @@ xfce_desktop_delete_event(GtkWidget *w,
     return TRUE;
 }
 
-static gboolean
-style_refresh_cb(gpointer *w)
-{
-    XfceDesktop *desktop = XFCE_DESKTOP(w);
-
-    xfce_desktop_refresh(desktop);
-    desktop->priv->style_refresh_timer = 0;
-
-    return FALSE;
-}
-
 static void
 xfce_desktop_style_set(GtkWidget *w, GtkStyle *old_style)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(w);
+#ifdef ENABLE_DESKTOP_ICONS
+    gdouble old_font_size;
+#endif
 
-    if(desktop->priv->style_refresh_timer != 0)
-        g_source_remove(desktop->priv->style_refresh_timer);
+    TRACE("entering");
 
-    desktop->priv->style_refresh_timer = g_timeout_add_seconds(1, (GSourceFunc)style_refresh_cb, w);
+    g_return_if_fail(XFCE_IS_DESKTOP(desktop));
+
+    if(!gtk_widget_get_realized(GTK_WIDGET(desktop)))
+        return;
+
+    if(desktop->priv->workspaces == NULL)
+        return;
+
+    if(GDK_IS_WINDOW(desktop->priv->bg_pixmap))
+        gdk_window_set_back_pixmap(gtk_widget_get_window(GTK_WIDGET(desktop)), desktop->priv->bg_pixmap, FALSE);
+
+    gtk_widget_queue_draw(GTK_WIDGET(desktop));
+
+#ifdef ENABLE_DESKTOP_ICONS
+    if(!desktop->priv->icon_view || !XFDESKTOP_IS_ICON_VIEW(desktop->priv->icon_view))
+        return;
+
+    /* reset the icon view style */
+    gtk_widget_set_style(desktop->priv->icon_view, NULL);
+    gtk_widget_reset_rc_styles(desktop->priv->icon_view);
+
+    old_font_size = desktop->priv->system_font_size;
+    if(xfce_desktop_ensure_system_font_size(desktop) != old_font_size
+       && desktop->priv->icon_view && !desktop->priv->icons_font_size_set)
+    {
+        xfdesktop_icon_view_set_font_size(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
+                                          desktop->priv->system_font_size);
+    }
+#endif
 }
 
 static void
@@ -1657,11 +1678,17 @@ xfce_desktop_refresh(XfceDesktop *desktop)
 {
     gint i, current_workspace;
 
+    TRACE("entering");
+
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
 
     if(!gtk_widget_get_realized(GTK_WIDGET(desktop)))
         return;
 
+    if(desktop->priv->workspaces == NULL) {
+        return;
+    }
+
     current_workspace = xfce_desktop_get_current_workspace(desktop);
 
     /* reload backgrounds */
@@ -1681,6 +1708,7 @@ xfce_desktop_refresh(XfceDesktop *desktop)
     }
     xfce_desktop_setup_icon_view(desktop);
 #endif
+
 }
 
 void xfce_desktop_arrange_icons(XfceDesktop *desktop)
