@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -40,6 +40,10 @@ struct _XfdesktopIconPrivate
     GdkRectangle pixbuf_extents;
     GdkRectangle text_extents;
     GdkRectangle total_extents;
+
+    GdkPixbuf *pix, *tooltip_pix;
+    gint cur_pix_width, cur_pix_height;
+    gint cur_tooltip_pix_width, cur_tooltip_pix_height;
 };
 
 enum {
@@ -54,6 +58,7 @@ enum {
 
 static guint __signals[SIG_N_SIGNALS] = { 0, };
 
+static void xfdesktop_icon_finalize(GObject *obj);
 
 G_DEFINE_ABSTRACT_TYPE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
 
@@ -61,8 +66,12 @@ G_DEFINE_ABSTRACT_TYPE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
 static void
 xfdesktop_icon_class_init(XfdesktopIconClass *klass)
 {
+    GObjectClass *gobject_class = (GObjectClass *)klass;
+
     g_type_class_add_private(klass, sizeof(XfdesktopIconPrivate));
-    
+
+    gobject_class->finalize = xfdesktop_icon_finalize;
+
     __signals[SIG_PIXBUF_CHANGED] = g_signal_new("pixbuf-changed",
                                                  XFDESKTOP_TYPE_ICON,
                                                  G_SIGNAL_RUN_LAST,
@@ -117,6 +126,13 @@ xfdesktop_icon_init(XfdesktopIcon *icon)
                                              XfdesktopIconPrivate);
 }
 
+static void
+xfdesktop_icon_finalize(GObject *obj)
+{
+    XfdesktopIcon *icon = XFDESKTOP_ICON(obj);
+
+    xfdesktop_icon_invalidate_pixbuf(icon);
+}
 
 void
 xfdesktop_icon_set_position(XfdesktopIcon *icon,
@@ -179,15 +195,26 @@ xfdesktop_icon_get_extents(XfdesktopIcon *icon,
 /*< required >*/
 GdkPixbuf *
 xfdesktop_icon_peek_pixbuf(XfdesktopIcon *icon,
-                           gint size)
+                           gint width, gint height)
 {
     XfdesktopIconClass *klass;
     
     g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
     klass = XFDESKTOP_ICON_GET_CLASS(icon);
     g_return_val_if_fail(klass->peek_pixbuf, NULL);
-    
-    return klass->peek_pixbuf(icon, size);
+
+    if(width != icon->priv->cur_pix_width || height != icon->priv->cur_pix_height)
+        xfdesktop_icon_invalidate_regular_pixbuf(icon);
+
+    if(icon->priv->pix == NULL) {
+        icon->priv->cur_pix_width = width;
+        icon->priv->cur_pix_height = height;
+
+        /* Generate a new pixbuf */
+        icon->priv->pix = klass->peek_pixbuf(icon, width, height);
+    }
+
+    return icon->priv->pix;
 }
 
 /*< required >*/
@@ -203,6 +230,22 @@ xfdesktop_icon_peek_label(XfdesktopIcon *icon)
     return klass->peek_label(icon);
 }
 
+/*< required >*/
+gchar *
+xfdesktop_icon_get_identifier(XfdesktopIcon *icon)
+{
+    XfdesktopIconClass *klass;
+
+    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+
+    if(!klass->get_identifier)
+        return NULL;
+
+    return klass->get_identifier(icon);
+}
+
 /*< optional; drags aren't allowed if not provided >*/
 GdkDragAction
 xfdesktop_icon_get_allowed_drag_actions(XfdesktopIcon *icon)
@@ -254,6 +297,31 @@ xfdesktop_icon_do_drop_dest(XfdesktopIcon *icon,
     return klass->do_drop_dest(icon, src_icon, action);
 }
 
+/*< optional >*/
+GdkPixbuf *
+xfdesktop_icon_peek_tooltip_pixbuf(XfdesktopIcon *icon,
+                                   gint width, gint height)
+{
+    XfdesktopIconClass *klass;
+
+    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+    klass = XFDESKTOP_ICON_GET_CLASS(icon);
+    g_return_val_if_fail(klass->peek_tooltip_pixbuf, NULL);
+
+    if(width != icon->priv->cur_tooltip_pix_width || height != icon->priv->cur_tooltip_pix_height)
+        xfdesktop_icon_invalidate_tooltip_pixbuf(icon);
+
+    if(icon->priv->tooltip_pix == NULL) {
+        icon->priv->cur_tooltip_pix_width = width;
+        icon->priv->cur_tooltip_pix_height = height;
+
+        /* Generate a new pixbuf */
+        icon->priv->tooltip_pix = klass->peek_tooltip_pixbuf(icon, width, height);
+    }
+
+    return icon->priv->tooltip_pix;
+}
+
 /*< optional >*/
 G_CONST_RETURN gchar *
 xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon)
@@ -270,6 +338,7 @@ xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon)
     return klass->peek_tooltip(icon);
 }
 
+
 /*< optional >*/
 void xfdesktop_icon_delete_thumbnail(XfdesktopIcon *icon)
 {
@@ -325,6 +394,31 @@ xfdesktop_icon_peek_icon_view(XfdesktopIcon *icon)
     return g_object_get_data(G_OBJECT(icon), "--xfdesktop-icon-view");
 }
 
+void
+xfdesktop_icon_invalidate_regular_pixbuf(XfdesktopIcon *icon)
+{
+    if(icon->priv->pix) {
+        g_object_unref(G_OBJECT(icon->priv->pix));
+        icon->priv->pix = NULL;
+    }
+}
+
+void
+xfdesktop_icon_invalidate_tooltip_pixbuf(XfdesktopIcon *icon)
+{
+    if(icon->priv->tooltip_pix) {
+        g_object_unref(G_OBJECT(icon->priv->tooltip_pix));
+        icon->priv->tooltip_pix = NULL;
+    }
+}
+
+void
+xfdesktop_icon_invalidate_pixbuf(XfdesktopIcon *icon)
+{
+    xfdesktop_icon_invalidate_regular_pixbuf(icon);
+    xfdesktop_icon_invalidate_tooltip_pixbuf(icon);
+}
+
 /*< signal triggers >*/
 
 void
